<?php

	define("MAILER_CRLF","\n");
	define("MAILER_LINELENGTH",75);
	define("MAILER_ID","SimpleMailer Version 1.0 [www.projects-and-software.de]");
	
	define("MAILER_ENC_QP",1);
	define("MAILER_ENC_BASE64",2);
	define("MAILER_ENC_8BIT",3);
	define("MAILER_ENC_7BIT",4);
	
	define("MAILER_INTERNAL",TRUE);
	

	class Mailer	{
		
		var $ENCODERS = array(		MAILER_ENC_QP		=> "quoted-printable",			
									MAILER_ENC_BASE64	=> "base64",
									MAILER_ENC_8BIT		=> "8-bit",
									MAILER_ENC_7BIT		=> "7-bit"				);
				
		
		var $arrReceivers = array();
		var $arrCCs = array();
		var $arrBCCs = array();
		
		// Sender information
		var $strSenderRealname = "";
		var $strSenderEmail = "";
		var $strReturnPath = "";
		var $strReplyTo = "";
		
		// Header vars
		var $strCharSet = "iso-8859-1";
		var $strContentType = "";
		var $iEncodingType = MAILER_ENC_8BIT;
		var $strServerName = "";
		var $strUniqueId = "";
		
		var $arrHeaderFields = array();
		var $strEncodedHeader = "";
		var $strOptMailParams = "";
		
		var $strBody = "";
		var $strEncodedBody = "";
		
		var $strSubject = "";
		
		
		//
		// Access methods
		//
		function AddReceiver($strEmail, $strName="")	{
			//$this->arrReceivers[] = $strEmail;
			$this->arrReceivers[] = $this->FormatAddress($strEmail,$strName);
			return TRUE;
		}
		
		function AddCC($strEmail, $strName="")	{		
			$this->arrCCs[] = $this->FormatAddress($strEmail,$strName);
			return TRUE;			
		}
		
		function AddBCC($strEmail, $strName="")	{
			$this->arrBCCs[] = $this->FormatAddress($strEmail,$strName);
			return TRUE;			
		}
		
		function AddHeaderField($strTitle, $strValue="")	{
			$this->arrHeaderFields[$strTitle] = $strValue;
			return TRUE;
		}
		
		function SetSubject($strSubject)	{
			$this->strSubject = $this->EncodeHeader($strSubject);
			return TRUE;
		}
		
		function SetBody($strBody)	{
			$this->strBody = $strBody;
		}
		
		function SetSender($strEmail,$strRealname="")	{
			$this->strSenderEmail = $strEmail;
			$this->strSenderRealname = $strRealname;
			return TRUE;
		}
		
		
		
		//
		// Constructor
		//
		function Mailer()	{	
			$this->ReadConfig();
			$this->Init();	
		}
		
		
		//
		// Public methods
		//
		function Send()		{
			echo "Class 'Mailer' should be treated as abstract. No sending action defined.";
			return FALSE;			
		}
		

		//
		// Protected methods
		//
		function Init()	{
			$this->strUniqueId = md5(microtime(TRUE));	
		}
		
		function ReadConfig()	{
			$this->strSenderEmail = GetConfig('ML_SENDER_ADDR');
			$this->strSenderRealname = GetConfig('ML_SENDER_NAME');
			$this->strServerName = GetConfig('ML_DOMAIN');
			return TRUE;			
		}
		
		function EncodeContent($strContent,$iEncType)	{
			switch($iEncType)	{
				case MAILER_ENC_BASE64:
					return (chunk_split(base64_encode($strContent),MAILER_LINELENGTH,MAILER_CRLF));
					break;
				case MAILER_ENC_QP:
					return ($this->EncodeQuotedPrintable($strContent));		
					break;
				case MAILER_ENC_7BIT: case MAILER_ENC_8BIT:
					return ($strContent);
					break;
			}
			return $strContent;			
		}
		
		
		function EncodeQuotedPrintable($strText,$bImap8BitEmulate=TRUE,$bWordWrap=TRUE)		{
			
			// split text into lines
			$arrLines=explode(chr(13).chr(10),$strText);

			for ($i=0; $i<count($arrLines); $i++) {
				$strLine =& $arrLines[$i];
   			
				if (strlen($strLine)===0) continue;

				// Deleted \x20
				$strRegExp = '/[^\x09\x21-\x3C\x3E-\x7E]/e';
				if($bImap8BitEmulate)
					$strRegExp = '/[^\x21-\x3C\x3E-\x7E]/e';

				$strReplacement = 'sprintf( "=%02X", ord ( "$0" ) ) ;';
				$strLine = preg_replace( $strRegExp, $strReplacement, $strLine );  
				{
					$iLength = strlen($strLine);
					$iLastChar = ord($strLine{$iLength-1});
					if(!($bImap8BitEmulate && ($i==count($arrLines)-1)))	{
						if(($iLastChar==0x09)||($iLastChar==0x20)) {
							$strLine{$iLength-1}='=';
							$strLine .= ($iLastChar==0x09)?'09':'20';
						}
					}
				} 	   
				if($bImap8BitEmulate) {
					$strLine=str_replace(' =0D','=20=0D',$strLine);
				}
				preg_match_all( '/.{1,73}([^=]{0,2})?/', $strLine, $arrMatch );
				
				if($bWordWrap)	{
					$strLine = implode( '=' . chr(13).chr(10), $arrMatch[0] );
				} 
			}
			return (implode(chr(13).chr(10),$arrLines));			
		}

		
		function DateTimeToRFC()	{
			// Use PHP encoding according to RFC2822
			$strTime = date("r");
			return $strTime;
		}
		
		
		function GetMsgId()	{
			return (sprintf("<%s@%s>",$this->strUniqueId,$this->strServerName));
		}

		// Encode headers as specified in RFC2047
		function EncodeHeader($strContent,$strCharset="",$iEncoding=MAILER_ENC_QP)	{
			if(empty($strCharset))	{ $strCharset = $this->strCharSet; }
			
			$strEnc = "=?".$strCharset."?".
				($iEncoding==MAILER_ENC_QP?"Q?":"B?");
			
			if($iEncoding==MAILER_ENC_QP)	{
				$strEnc.= $this->EncodeQuotedPrintable($strContent,TRUE,FALSE);
			} else {
				$strEnc.= (chunk_split(base64_encode($strContent),MAILER_LINELENGTH,MAILER_CRLF));				
			}
					
			$strEnc.= "?=";
			return $strEnc;			
		}

		
		function FormatAddress($strEmail,$strName="",$bInternal=MAILER_INTERNAL)	{
			if(empty($strName))		{
				return ($strEmail);
			} else {
				if($bInternal)	{
					// Due to an everlasting bug in PHP-mail() its not possible to 
					// send encoded receiver headers to an mail server
					return ($strEmail);					
				} else {
					return $this->EncodeHeader($strName)." <".$strEmail.">";				
				}
			}
		}
		
		
		function SetHeaderFields()	{
			
			// MIME version
			$this->AddHeaderField("MIME-Version","1.0");
						
			// Add mandatory fields
			$this->AddHeaderField("Date",$this->DateTimeToRFC());
			
			if(empty($this->strReturnPath))	{
				$this->AddHeaderField("Return-Path",$this->strSenderEmail);
			}
			
			// Add all kind of receivers (regular, copy, blind)			
			if(sizeof($this->arrReceivers)>0)	{
				$this->AddHeaderField("To",implode(", ",$this->arrReceivers));	
			}
			
			if(sizeof($this->arrCCs)>0)		{
				$this->AddHeaderField("Cc",implode(", ",$this->arrCCs));				
			}
			
			if(sizeof($this->arrBCCs)>0)	{
				$this->AddHeaderField("Bcc",implode(", ",$this->arrBCCs));
			}
			
			// Only blind copy receivers, add dummy receiver			
			if(sizeof($this->arrReceivers)==0 && sizeof($this->arrCCs)==0)	{
				$this->AddHeaderField("To","undisclosed-recipients:;");
			} 
			
			// Reply-To
			if(!empty($this->strReplyTo))	{
				$this->AddHeaderField("Reply-To",$this->strReplyTo);
			}
			
			// From
			$this->AddHeaderField("From",$this->FormatAddress($this->strSenderEmail,
				$this->strSenderRealname,MAILER_INTERNAL));
			
			// Message Id
			$this->AddHeaderField("Message-ID",$this->GetMsgId());
			
			// Mailer
			$this->AddHeaderField("X-Mailer",MAILER_ID);
			
			return TRUE;		
		}
		
		
		function BuildHeader($arrHeaderFields)	{

			$strEncodedHeader = "";
			if(is_array($arrHeaderFields))	{
				foreach($arrHeaderFields as $strKey=>$strContent)	{
					$strEncodedHeader.= $strKey.": ".$strContent.MAILER_CRLF;
				}
			}
			// Delimit header by adding to line breaks
			$strEncodedHeader.= MAILER_CRLF . MAILER_CRLF ;
			
			return $strEncodedHeader;		
		}
	}

	
	
	class MailerPlain extends Mailer 	{
		
		function MailerPlain()	{
			parent::Mailer();
			
			$this->strCharSet = "iso-8859-1";
			$this->strContentType = "text/plain";
			$this->iEncodingType = MAILER_ENC_8BIT;
		}
		
		function Send()		{
			
			$this->SetHeaderFields();
			$this->strEncodedHeader = $this->BuildHeader($this->arrHeaderFields);
			$this->strEncodedBody = $this->EncodeContent($this->strBody,$this->iEncodingType);
			
			$strReceiver = implode(", ",$this->arrReceivers);
			
			if(ini_get("safe_mode")==1 || empty($this->strOptMailParams))	{ 			
				$bRetVal = @mail($strReceiver,$this->strSubject,$this->strEncodedBody,$this->strEncodedHeader);
			} else {
				$bRetVal = @mail($strReceiver,$this->strSubject,$this->strEncodedBody,$this->strEncodedHeader,
					$this->strOptMailParams);
			}
						
			return $bRetVal;
		}
		
		function SetHeaderFields()	{
			
			parent::SetHeaderFields();
			
			// Add headers for simple plain mail
			$this->AddHeaderField("Content-Transfer-Encoding", $this->ENCODERS[$this->iEncodingType] );
			$this->AddHeaderField("Content-Type", sprintf("%s; charset=\"%s\"",$this->strContentType,$this->strCharSet));
			
			return TRUE;
		}
		
	}

	
	class MailerHtml extends MailerPlain 	{
		
		function MailerHtml()	{
			parent::MailerPlain();
			
			// Set parameters according to a plain email	
			$this->strCharSet = "iso-8859-1";
			$this->strContentType = "text/html";
			$this->iEncodingType = MAILER_ENC_QP;
		}
	}
	
?>