<?php

	/*
		Include mailer and mailtemplate-engine
	*/
	require_once("inc/mailer.class.php");
	require_once("inc/mailtemplate.class.php");

	
	define("PO_MAILER_HTML",1);
	define("PO_MAILER_PLAIN",2);
	
	
	class PostOffice	{
		
		var $oTemplate = NULL;
		var $oMailer = NULL;
		var $strLanguage = "GER";
		
		var $iMailType = PO_MAILER_HTML;
		
		var $strLastError = "";
		
		/*
			Constructor
		*/
		function PostOffice()	{
			$this->Init();
		}
				
		
		/*
			Access methods
		*/
		function GetLastError()	{
			return $this->strLastError;
		}
		
		function SetLanguage($strLanguage)	{
			$this->strLanguage = $strLanguage;
			return TRUE;
		}
		
		
		/*
			Protected methods
		*/
		function CreateMailer($iType=PO_MAILER_HTML)	{
			switch($iType)	{
				case PO_MAILER_PLAIN:
					$this->oMailer = new MailerPlain();
					break;
				default:
					$this->oMailer = new MailerHtml();
					break;
			}
			return TRUE;
		}

		
		function CreateTemplate()	{
			$this->oTemplate = new MailTemplate();	
			$this->SetDefaults();			
			return TRUE;		
		}
		
		
		function ParseTemplate($strPathFile)	{

			// Set and check template file
			if(!$this->oTemplate->SetTemplate($strPathFile))	{
				$this->strLastError = "Error reading template (".$strPathFile.")";
				return FALSE;
			}
			
			// Read and parse template file/content
			if(!$this->oTemplate->ParseContent())	{
				$this->strLastError = "Error parsing content.";
				return FALSE;
			}
			return TRUE;			
		}
		
		
		function SetDefaults()		{
			if(is_null($this->oTemplate))	{		
				return FALSE;		
			}
			
			$arrSets = array(	"DomainName" 	=> GetConfig('PD_WEBURL'),
								"IndexLink"		=> "http://".GetConfig('PD_WEBURL')."/".
													GetConfig('PD_SCRIPTPATH')."index.php",
								"NewAdLink"		=> "http://".GetConfig('PD_WEBURL')."/".
													GetConfig('PD_SCRIPTPATH')."newad.php",
								"AdminLink"		=> "http://".GetConfig('PD_WEBURL')."/".
													GetConfig('PD_SCRIPTPATH')."admin.php",
								"PicActiveLink"	=> "http://".GetConfig('PD_WEBURL')."/".
													GetConfig('PD_SCRIPTPATH')."admin_picturerelease.php",
								"DateTime"		=> FormatDateDt(mktime(),FALSE),
								"IpAddress"		=> $_SERVER['REMOTE_ADDR'],
								"Version"		=> GetConfig('VERSION'),
								"Program"		=> "cts classAd",
								"ProviderShortInfo"	=> GetConfig('ML_PROVIDER_SHORT_INFO')
			);							
			$this->oTemplate->SetGroup($arrSets);										
			return TRUE;
		}
		
		function Init()		{
			$this->strLanguage = GetConfig('DEFMAILLANG');
			
			// Create template engine, set defaults and substitutions
			$this->CreateTemplate();
			
			return TRUE;
		}
		
		/*
			Abstract methods
		*/
		function Send()	{}
		
		function GetTemplateName($strFileName)		{
			$strTemplate = GetConfig('PD_EMAILTEMPLATES').
				($this->iMailType==PO_MAILER_PLAIN?"plain":"html")."/".$strFileName.".".
				$this->strLanguage.".tpl";
			return $strTemplate;
		}
	}

	
	class PostOfficeUserOptIn extends PostOffice 	{
		
		var $iUserId = -1;
		var $arrUserData = array();
				
		
		function PostOfficeUserOptIn($iUserId)	{
			parent::PostOffice();
			$this->iUserId = $iUserId;
		}

		
		function Send()	{
			
			// Set user release code
			if(!SetUserReleaseCode($this->iUserId))	{
				$this->strLastError = "Unable to insert release code.";
				return FALSE;
			}
			
			// Get user data
			if(!$this->GetUserData())	{
				return FALSE;
			}
	
			$this->SetTemplateData();
			
			// Decide about mailer
			if($this->arrUserData['useremailhtml'])		{		
				$this->iMailType = PO_MAILER_HTML;		
			} else {
				$this->iMailType = PO_MAILER_PLAIN;
			}
			
			// Create mailer
			$this->CreateMailer($this->iMailType);
						
			// Get template name
			if(!$this->ParseTemplate($this->GetTemplateName("user_optin")))	{
				return FALSE;
			}
			
			// Set mailer
			$this->oMailer->AddReceiver($this->arrUserData['useremail']);
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_USEROPTINMAILSUBJECT,GetConfig('PD_WEBURL')));
			$this->oMailer->Send();

			return TRUE;		
		}
		
		/*
			Protected methods
		*/	
		function GetUserData()	{
			$this->arrUserData = GetUserDetails($this->iUserId);
			if(!is_array($this->arrUserData) || sizeof($this->arrUserData)==0)	{
				$this->strLastError = "Unable to read userdata.";
				return FALSE;
			}
			return TRUE;	
		}
		
		
		function SetTemplateData()	{
			
			SetUserInfoToTemplate($this->oTemplate,$this->arrUserData);
								
			$this->oTemplate->Set("ReleaseLinkComplete","http://".GetConfig('PD_WEBURL')."/".
				GetConfig('PD_SCRIPTPATH')."releaseuser.php?userid=".$this->iUserId.
				"&seccode=".$this->arrUserData['userreleasecode']);
			$this->oTemplate->Set("ReleaseCode",$this->arrUserData['userreleasecode']);
			$this->oTemplate->Set("ReleaseLink","http://".GetConfig('PD_WEBURL')."/".
				GetConfig('PD_SCRIPTPATH')."releaseuser.php");			
			
			return TRUE;
		}
		
	}
	
	
	class PostOfficeAdContact extends PostOffice {
		
		var $arrAdData = array();
		var $arrUserData = array();
		
		var $iAdId = -1;
		var $iUserId = -1;
		
		
		function PostOfficeAdContact($iAdId)	{
			parent::PostOffice();
			$this->iAdId = $iAdId;
		}

		
		function SetContact($arrContact)	{
			$this->oTemplate->SetGroup($arrContact);
			return TRUE;
		}
		
		
		function Send()		{
		
			// Get necessary information
			$this->GetAdData($this->iAdId);
			$this->GetUserData($this->arrAdData['aduserid']);
					
			SetUserInfoToTemplate($this->oTemplate,$this->arrUserData);
			SetAdInfoTemplate($this->oTemplate,$this->arrAdData);
			
			// Take default mailer, because we have no information about receiver's preferences
			if(GetConfig('ML_EMAIL_HTML'))		{		
				$this->iMailType = PO_MAILER_HTML;		
			} else {
				$this->iMailType = PO_MAILER_PLAIN;
			}
			$this->CreateMailer($this->iMailType);
						
			// Get template name
			if(!$this->ParseTemplate($this->GetTemplateName("response")))	{
				return FALSE;
			}			
						
			// Set mailer
			$this->oMailer->AddReceiver($this->arrAdData['ademail']);
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_CONTMAILSUBJECT,GetConfig('PD_WEBURL')));
			$this->oMailer->Send();

			return TRUE;		
		}
		
		
		function GetUserData($iUserId)	{
			$this->arrUserData = GetUserDetails($iUserId);
			return TRUE;
		}

		
		function GetAdData($iAdId)	{
			$data = GetArticleByID($iAdId);
			$this->arrAdData = $data[0];
			return TRUE;
		}
		
	}
	
	
	class PostOfficeTellAd extends PostOffice {
		
		var $iAdId = -1;
		
		var $arrAdData = array();
		var $arrTellerData = array();

		
		function PostOfficeTellAd($iAdId)	{
			parent::PostOffice();
			$this->iAdId = $iAdId;
		}
		
		function SetTeller($arrTellerData)	{
			// Double storage for easier access to necessary data
			$this->arrTellerData = $arrTellerData;
			$this->oTemplate->SetGroup($arrTellerData);
			return TRUE;
		}
		
		function Send()		{
			
			$this->GetAdData($this->iAdId);
			SetAdInfoTemplate($this->oTemplate,$this->arrAdData);
			
			// Take default mailer, because we have no information about receiver's preferences
			if(GetConfig('ML_EMAIL_HTML'))		{		
				$this->iMailType = PO_MAILER_HTML;		
			} else {
				$this->iMailType = PO_MAILER_PLAIN;
			}
			$this->CreateMailer($this->iMailType);			
			
			// Get template name
			if(!$this->ParseTemplate($this->GetTemplateName("tellad")))	{
				return FALSE;
			}			
						
			// Set mailer
			$this->oMailer->AddReceiver($this->arrTellerData['FriendEmail']);
			$this->oMailer->SetSender($this->arrTellerData['AdTellerMail'],$this->arrTellerData['AdTellerName']);
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_TELLADSUBJECT, GetConfig('PD_WEBURL')));
			$this->oMailer->Send();

			return TRUE;		
		}
		
		
		function GetAdData($iAdId)	{
			$data = GetArticleByID($iAdId);
			$this->arrAdData = $data[0];
			return TRUE;
		}
		
	}
	
	
	class PostOfficeLostPassword extends PostOffice {
		
		var $iUserId = -1;
		var $arrUserData = array();
		var $strNewPassword = "";
		
		
		function PostOfficeLostPassword($iUserId)	{
			parent::PostOffice();
			$this->iUserId = $iUserId;
		}

		function SetPassword($strPassword)	{
			$this->strNewPassword = $strPassword;
			return TRUE;
		}
		
		function Send()		{
			
			// Get user data
			$this->GetUserData($this->iUserId);
			
			// Create mailer
			$this->iMailType = $this->arrUserData['useremailhtml']?PO_MAILER_HTML:PO_MAILER_PLAIN;
			$this->CreateMailer($this->iMailType);		
			
			SetUserInfoToTemplate($this->oTemplate,$this->arrUserData);
			$this->oTemplate->Set("NEWPASSWORD",$this->strNewPassword);
			
			// Parse Template
			if(!$this->ParseTemplate($this->GetTemplateName("lostpassword")))	{
				$this->strLastError = "Error parsing template 'lostpassword'!";
				return FALSE;
			}
			
			// Set mailer and send mail
			$this->oMailer->AddReceiver($this->arrUserData['useremail']);
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_LOSTPWMAILSUBJECT,GetConfig('PD_WEBURL')));
			$bSuccess = $this->oMailer->Send();
						
			return $bSuccess;
		}
		
		
		function GetUserData($iUserId)	{
			$this->arrUserData = GetUserDetails($iUserId);
			return TRUE;
		}
		
	}

	
	class PostOfficeAdInfo extends PostOffice {
		
		var $iAdId = -1;
		var $arrAdData = array();
		var $oPicList = NULL;
		
		var $strReleaseLink = "";
		
		
		function PostOfficeAdInfo($iAdId=-1)	{
			parent::PostOffice();
			$this->iAdId = $iAdId;
		}
		
		
		function SetReleaseLink($strLink)	{
			$this->strReleaseLink = $strLink;
			return TRUE;		
		}

		
		function Send()		{
			
			/*
				Create mailer
			*/
			$this->iMailType = GetConfig('ML_EMAIL_HTML')?PO_MAILER_HTML:PO_MAILER_PLAIN;
			$this->CreateMailer($this->iMailType);			
			
			/*
				Get all necessary data and set template vars
			*/
			$this->GetAdData($this->iAdId);
			$this->GetPictureList($this->iAdId);
			
			SetAdInfoTemplate($this->oTemplate,$this->arrAdData);
			SetPicListInfoTemplate($this->oTemplate,$this->oPicList);
			
			$this->oTemplate->Set("RELEASELINK",$this->strReleaseLink);
			
			// Parse Template
			if(!$this->ParseTemplate($this->GetTemplateName("releasesubmit")))	{
				$this->strLastError = "Error parsing template 'releasesubmit'!";
				return FALSE;
			}
			
			// Set mailer and send mail
			$this->oMailer->AddReceiver(GetConfig("ML_ADMINEMAIL"));
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_RELADSMAILSUBJECT,GetConfig('PD_WEBURL')));
			$this->oMailer->Send();
						
			return TRUE;
		}
		
			
		/*
			Helper functions
		*/
		function GetAdData($iAdId)	{
			$data = GetArticleByID($iAdId);
			$this->arrAdData = $data[0];
			return TRUE;
		}
		
		
		function GetPictureList($iAdId=-1)	{
			
			if($iAdId<=0)	{
				$this->oPicList = NULL;
				return FALSE;
			}

			$this->oPicList = new PictureList();
			$this->oPicList->ReadList($iAdId);
			
			return TRUE;
		}
		
	}
	
	
	class PostOfficeContact extends PostOffice {
		
		var $arrContacts = array();
		
		
		function PostOfficeContact($arrForm)	{
			parent::PostOffice();
			$this->arrContacts = $arrForm;
		}
		
		
		function Send()		{
			
			// Create mailer
			$this->iMailType = GetConfig('ML_EMAIL_HTML')?PO_MAILER_HTML:PO_MAILER_PLAIN;
			$this->CreateMailer($this->iMailType);		
			
			// Set template
			$this->oTemplate->SetGroup($this->arrContacts);
			
			
			// Parse template
			// Parse Template
			if(!$this->ParseTemplate($this->GetTemplateName("contact")))	{
				$this->strLastError = "Error parsing template 'contact'!";
				return FALSE;
			}
			
			// Set mailer and send mail
			$this->oMailer->AddReceiver(GetConfig("ML_ADMINEMAIL"));
			$this->oMailer->SetBody($this->oTemplate->GetContent());
			$this->oMailer->SetSubject(ParseIn(LANG_GENERALCONTMAILSUBJECT,GetConfig('PD_WEBURL')));
			$this->oMailer->Send();
						
			return TRUE;			
		}
		
	}
	
?>